/**
 * <a href="http://www.openolat.org">
 * OpenOLAT - Online Learning and Training</a><br>
 * <p>
 * Licensed under the Apache License, Version 2.0 (the "License"); <br>
 * you may not use this file except in compliance with the License.<br>
 * You may obtain a copy of the License at the
 * <a href="http://www.apache.org/licenses/LICENSE-2.0">Apache homepage</a>
 * <p>
 * Unless required by applicable law or agreed to in writing,<br>
 * software distributed under the License is distributed on an "AS IS" BASIS, <br>
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. <br>
 * See the License for the specific language governing permissions and <br>
 * limitations under the License.
 * <p>
 * Initial code contributed and copyrighted by<br>
 * frentix GmbH, http://www.frentix.com
 * <p>
 */
package org.olat.modules.qpool.ui;

import org.olat.core.CoreSpringFactory;
import org.olat.core.gui.UserRequest;
import org.olat.core.gui.components.Component;
import org.olat.core.gui.components.panel.Panel;
import org.olat.core.gui.components.velocity.VelocityContainer;
import org.olat.core.gui.control.Controller;
import org.olat.core.gui.control.Event;
import org.olat.core.gui.control.WindowControl;
import org.olat.core.gui.control.controller.BasicController;
import org.olat.modules.qpool.QuestionItem;
import org.olat.modules.qpool.QuestionPoolModule;
import org.olat.modules.qpool.QPoolSPI;

/**
 * 
 * Initial date: 12.02.2013<br>
 * @author srosse, stephane.rosse@frentix.com, http://www.frentix.com
 *
 */
public class QuestionItemPreviewController extends BasicController {

	private QuestionItem item;
	private Controller previewCtrl;
	private final Panel previewPanel;
	private final QuestionPoolModule poolModule;
	
	public QuestionItemPreviewController(UserRequest ureq, WindowControl wControl) {
		super(ureq, wControl);
		
		poolModule = CoreSpringFactory.getImpl(QuestionPoolModule.class);
		previewPanel = new Panel("preview.container");
		putInitialPanel(previewPanel);
	}
	
	@Override
	protected void doDispose() {
		//
	}

	public QuestionItem getItem() {
		return item;
	}
	
	public void reset() {
		this.item = null;
		removeAsListenerAndDispose(previewCtrl);
		previewCtrl = null;
		previewPanel.setContent(null);
	}
	
	public void updateItem(UserRequest ureq, QuestionItem item) {
		this.item = item;
		removeAsListenerAndDispose(previewCtrl);

		Component content;
		QPoolSPI spi = poolModule.getQuestionPoolProvider(item.getFormat());
		if(spi == null) {
			content = getRawContent(item);
		} else {
			previewCtrl = spi.getPreviewController(ureq, getWindowControl(), item);
			if(previewCtrl == null) {
				content = getRawContent(item);
			} else {
				listenTo(previewCtrl);
				content = previewCtrl.getInitialComponent();
			}
		}
		
		previewPanel.setContent(content);
	}
	
	private Component getRawContent(QuestionItem item) {
		VelocityContainer content = createVelocityContainer("raw_content");
		
		return content;
	}

	@Override
	protected void event(UserRequest ureq, Component source, Event event) {
		//
	}
}
